WITH   Ada.Unchecked_Conversion ;
WITH   Interfaces.C ;
WITH   Stdarg ;
WITH   System ;
WITH   Win32 ;
WITH   Win32.Crt.Strings ;
WITH   Win32.Windef ;
WITH   Win32.Wingdi ;
WITH   Win32.Winuser ;

PACKAGE BODY   CharTb24_Pkg  IS

   FUNCTION   CP   ( I_Str : Win32.CHAR_Array )  RETURN  Win32.LPCSTR  IS
      FUNCTION   UC  IS
         new Ada.Unchecked_Conversion ( System.Address , Win32.LPCSTR );
   BEGIN
      RETURN  UC ( I_Str(I_Str'First)'Address );
   END CP;

   FUNCTION   CPS   ( I_Str : Win32.CHAR_Array )  RETURN  Win32.LPSTR  IS
      FUNCTION   UC  IS
         new Ada.Unchecked_Conversion ( System.Address , Win32.LPSTR );
   BEGIN
      RETURN  UC ( I_Str(I_Str'First)'Address );
   END CPS;

   FUNCTION   To_IntW  IS  new Ada.Unchecked_Conversion(Win32.PCHAR, Integer);
   FUNCTION   To_PCHAR  IS  new Ada.Unchecked_Conversion(Integer, Win32.PCHAR);

   FUNCTION   StPCpy   ( I_Dest : Win32.PSTR
                       ; I_Source : Win32.PCSTR )  RETURN  Win32.PSTR  IS
      L_IgnoredP : Win32.PSTR ;
      L_AddedLen : Standard.INTEGER ;
   BEGIN
      L_IgnoredP := Win32.Crt.Strings.StrCpy ( I_Dest , I_Source );
      L_AddedLen := Standard.INTEGER( Win32.Crt.Strings.StrLen(Win32.PCCH(I_Dest)) );
      RETURN  To_PCHAR(To_IntW(I_Dest) + L_AddedLen);
   END StPCPy;

   USE TYPE   Win32.CHAR_Array ;      --- To get "&"

   Nul : Win32.Char RENAMES Win32.Nul ;

   IgnoredI : Win32.INT ;
   IgnoredP : Win32.PSTR ;

   C_MainPMTitle : CONSTANT Win32.CHAR_Array := "Information"  &  Nul ;

   CRLF : CONSTANT Win32.CHAR_Array := Win32.Char(Character'Val(13))  &  Win32.Char(Character'Val(10));

   C_TitleLn1 : CONSTANT Win32.CHAR_Array := "CODE   CHAR"  &  CRLF  &  Nul ;
   C_TitleLn2 : CONSTANT Win32.CHAR_Array := "-----------"  &  CRLF  &  Nul ;
   C_NumberFmt : CONSTANT Win32.CHAR_Array := "%u"  &  Nul ;
   C_3SpacesZ : CONSTANT Win32.CHAR_Array := "   "  &  Nul ;
   CRLFz : CONSTANT Win32.CHAR_Array := Win32.Char(Character'Val(13))  &  Win32.Char(Character'Val(10))  &  Nul ;

   L_Buffer : ALIASED Win32.CHAR_Array(0..4096);
   L_Buf25  : ALIASED Win32.CHAR_Array(0..25);
   L_Buf01z : ALIASED Win32.CHAR_Array(0..1);

   PROCEDURE   ShowCharTb24  IS
      FUNCTION   "&"  IS  new Stdarg.Concat ( Win32.LONG );

      L_P : Win32.LPSTR ;
   BEGIN
      L_Buffer(0) := Nul ;
      L_P := CPS(L_Buffer);

      L_P := StPCpy ( L_P , CP(C_TitleLn1) );
      L_P := StPCpy ( L_P , CP(C_TitleLn2) );

      L_Buf01z(1) := Nul ;

      ----- LOOP , On each char
      FOR i in 0..255 LOOP
         IgnoredI := Win32.Winuser.wvsprintf ( CPS(L_Buf25) , CP(C_NumberFmt)
                                             , Stdarg.Empty & Win32.LONG(i) );
         L_P := StPCpy ( L_P , CP(L_Buf25) );
         L_P := StPCpy ( L_P , CP(C_3SpacesZ) );

         ----- Write char  ( Ignoring problem chars )
         CASE i IS
            WHEN 0  => L_Buf01z(0) := '-' ;      --- EOF   End Of File
            WHEN 9  => L_Buf01z(0) := '-' ;      --- HT    Tabulation
            WHEN 10 => L_Buf01z(0) := '-' ;      --- LF    Line Feed
            WHEN 13 => L_Buf01z(0) := '-' ;      --- CR    Carriage Return

            WHEN OTHERS =>
               L_Buf01z(0) := Win32.Char(Character'Val(i));
         END CASE;
         L_P := StPCpy ( L_P , CP(L_Buf01z) );

         L_P := StPCpy ( L_P , CP(CRLFz) );
      END LOOP;--i

      IgnoredI := Win32.Winuser.MessageBox ( System.Null_Address
                                           , CP(L_Buffer)
                                           , CP(C_MainPMTitle)
                                           , Win32.Winuser.MB_OK or Win32.Winuser.MB_ICONEXCLAMATION );
   END ShowCharTb24;
END CharTb24_Pkg;
